@@ -178,7 +178,7 @@ get_current_tz(void)
 	{
 		g_printerr ("Could not read timezone file (%s), using default.\n", err->message);
 		g_error_free (err);
-		result = g_strdup ("UTC");
+		result = g_strdup ("Europe/Berlin");
 	}
 	return result;  
 }
@@ -263,11 +263,18 @@ do_tabchange (GtkNotebook * notebook,
 gchar * 
 getSelectedTimezoneText (void)
 {
+	GtkTreeIter	curZoneIter;
 	gint		curIndex;
 	gint		curAreaIndex;
 	gchar		*retVal = NULL;
 	
-	curAreaIndex	= gtk_combo_box_get_active(GTK_COMBO_BOX(self.timezoneArea));
+	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(self.timezoneArea), &curZoneIter))
+    {
+        GtkTreeModel    *timezoneAreaModel;
+        timezoneAreaModel = gtk_combo_box_get_model(GTK_COMBO_BOX(self.timezoneArea));
+        gtk_tree_model_get(timezoneAreaModel, &curZoneIter, COL_VIEW_ID, &curAreaIndex, -1);
+    }
+    
 	curIndex	= gtk_combo_box_get_active(GTK_COMBO_BOX(self.timezone));
 	if ((curAreaIndex > -1) && (curIndex > -1))
 	{
@@ -281,109 +288,119 @@ void
 timezonearea_combo_changed(GtkComboBox *widget, gpointer user_data)
 {
 	GtkListStore	*timezoneModel;
-	GtkTreeIter	timezoneIter;
-	gint		curIndex;
+	GtkTreeIter	timezoneIter, curZoneIter;
+	gint		curIndex = -1;
 	gchar		**tzPointer;
 	guint		idx;
 
-	curIndex	= gtk_combo_box_get_active(GTK_COMBO_BOX(self.timezoneArea));
-	if (curIndex > -1)
+	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(self.timezoneArea), &curZoneIter))
+    {
+        GtkTreeModel    *timezoneAreaModel;
+        timezoneAreaModel = gtk_combo_box_get_model(GTK_COMBO_BOX(self.timezoneArea));
+        gtk_tree_model_get(timezoneAreaModel, &curZoneIter, COL_VIEW_ID, &curIndex, -1);
+    }
+    
+ 	if (curIndex > -1)
 	{
-		timezoneModel		= (GtkListStore *)gtk_combo_box_get_model(GTK_COMBO_BOX(self.timezone));
+		timezoneModel = (GtkListStore *)gtk_combo_box_get_model(GTK_COMBO_BOX(self.timezone));
 		/* clear previous timezone texts */
 		gtk_list_store_clear(timezoneModel);
 		
 		/* add new timezone texts */
 		idx			= 0;
-		tzPointer	= timezoneAreaArray[curIndex];		
-		while(*tzPointer != NULL) {
-       			gtk_list_store_append(timezoneModel, &timezoneIter);
-				gtk_list_store_set(timezoneModel, &timezoneIter,
-					COL_VIEW_ID, idx,
-					COL_VIEW_NAME, *tzPointer,
-					-1);
-			tzPointer++;
-			idx++;
-		}
+		tzPointer	= timezoneAreaArray[curIndex];
+        if (tzPointer)
+	    	while(tzPointer[idx] != NULL) {
+       	    		gtk_list_store_append(timezoneModel, &timezoneIter);
+		    		gtk_list_store_set(timezoneModel, &timezoneIter,
+		    			COL_VIEW_ID, idx,
+		    			COL_VIEW_NAME, tzPointer[idx],
+		    			-1);
+		    	idx++;
+		    }
 		/* select first from the list by default */
 		gtk_combo_box_set_active(GTK_COMBO_BOX (self.timezone), 0);
 	}	
 }
 
 /*
-* Method for finding timezone area index by using timezone name as a key.
-* Method could be optimized by only searching from "Other" area and
-* if not found from there then comparing the timezone area names with the start part of the
-* timezone area name. But this method is not done very often and this way it was easier to implement
-* for me so...
-*
-* If area is not found -1 is returned.
-*/
-gint getTimezoneAreaIndexByTimezoneName(gchar *tzNameParam)
+ * Find area name index from name string.
+ * If area is not found -1 is returned.
+ */
+gint 
+get_timezone_area_index (const gchar *tzNameParam)
 {
-	gint		retVal = -1;
-	gchar		**tzAreasPointer;
-	guint		idx = 0;
-	gboolean	curFound = FALSE;
-	gchar		*searchname;
+	gint		i = 0;
+	gchar		*searchname, *pos;
+    gint        retval = -1;
 
-	searchname = strstr (tzNameParam, "/") + 1;
-	
-	if ((gint)searchname == 1)
-		searchname = tzNameParam;
-
-	while (timezoneNameArray[idx] != NULL) 
+    
+	searchname = g_strdup (tzNameParam);
+    pos = strchr (searchname, '/');
+    if (pos)
+        pos[0] = 0;        
+    
+	while (timezoneNameArray[i] != NULL) 
 	{
-		tzAreasPointer	= timezoneAreaArray[idx];
-    	while (tzAreasPointer && (*tzAreasPointer != NULL)) 
-		{
-   			if (strstr(*tzAreasPointer, searchname)) 
-			{
-				retVal = idx;
-   				curFound = TRUE;
-   				break;
-   			}
-			tzAreasPointer++;
-		}
-		if (curFound == TRUE)
-		{
-			break;
-		}
-		idx++;
+        if (!strcmp (timezoneNameArray[i], searchname))
+        {
+            retval = i;
+            break;
+        }
+        i++;
 	}
-	return retVal;
+    g_free (searchname);
+ 
+	return retval;
 }
 
+gint 
+area_index_to_selector_index (gint i)
+{
+	gint j, retval = i;
+ 
+    for (j = 0; j <= i; j++)
+	{
+        if (!timezoneAreaArray[j])
+            retval--;
+	}
+    
+	return retval;
+}
+
+
 /**
 * If timezone is not found from the timezone area, -1 is returned.
 */
 gint 
-getTimezoneIndexByTimezoneAreaIndexAndTimezoneName(guint tzAreaIndexParam, gchar *tzNameParam)
+get_timezone_index (gint tzAreaIndexParam, gchar *tzNameParam)
 {
 	gint	retVal = -1;
 	gchar	**tzAreasPointer;
-	guint	idx = 0;
+	gint	idx = 0;
 	gchar	*searchname;
 
-	searchname = strstr (tzNameParam, "/") + 1;
+	if ((tzAreaIndexParam < 0) 
+        || (tzAreaIndexParam > timezoneAreaArray_len)
+        || !tzNameParam)
+        return retVal;
+
+	searchname = strstr (tzNameParam, "/");
 	
-	if ((gint)searchname == 1)
+	if (searchname)
+        searchname++;
+    else
 		searchname = tzNameParam;
 	
-	if (tzAreaIndexParam >= 0)
-	{
-    		tzAreasPointer	= timezoneAreaArray[tzAreaIndexParam];
-    		while (tzAreasPointer && (*tzAreasPointer != NULL))
-			{
-    			if (strstr(*tzAreasPointer, searchname)) 
-				{
-    				retVal	= idx;
-    				break;
-    			}
-				tzAreasPointer++;
-				idx++;
-			}
-	}
+    tzAreasPointer = timezoneAreaArray[tzAreaIndexParam];
+    
+    while (tzAreasPointer[idx] != NULL)
+    {
+	    if (!strcmp(tzAreasPointer[idx], searchname))
+            return idx;
+        idx++;
+    }
+    
 	return retVal;
 }
 
@@ -532,18 +549,16 @@ Time_Build_Objects(gboolean nonroot)
 	gtk_table_attach (GTK_TABLE (table), self.catlabel4, 
 	                  0, 4, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 			
-	// timezone area combobox
-	fstr	= g_strdup(get_current_tz());
-   	if ((fstr == NULL) || (strlen(fstr) <= 0))
-   	{
-     	fstr	= g_strdup("UTC");
-   	}
-	selTimezoneAreaIndx	= getTimezoneAreaIndexByTimezoneName(fstr);
+	/* timezone area combobox */
+	fstr = get_current_tz();
+    
+	selTimezoneAreaIndx	= get_timezone_area_index (fstr);
+
 	if (selTimezoneAreaIndx == -1)
 	{
 		g_free(fstr);
-		fstr = g_strdup("Europe/Helsinki");
-		selTimezoneAreaIndx	= getTimezoneAreaIndexByTimezoneName(fstr);
+		fstr = g_strdup("Europe/Berlin");
+		selTimezoneAreaIndx	= get_timezone_area_index (fstr);
 	}
 	if (selTimezoneAreaIndx == -1)
 	{
@@ -552,7 +567,7 @@ Time_Build_Objects(gboolean nonroot)
 	}
 	else
 	{
-		selTimezoneIndx	= getTimezoneIndexByTimezoneAreaIndexAndTimezoneName(selTimezoneAreaIndx, fstr);
+		selTimezoneIndx	= get_timezone_index (selTimezoneAreaIndx, fstr);
 	}
 	g_free(fstr);
 	
@@ -562,11 +577,14 @@ Time_Build_Objects(gboolean nonroot)
 	idx	= 0;
 	while (timezoneNameArray[idx] != NULL) 
 	{
-    	gtk_list_store_append(timezoneAreaModel, &timezoneAreaIter);
-	   	gtk_list_store_set(timezoneAreaModel, &timezoneAreaIter,
-               			COL_VIEW_ID, idx,
-                       	COL_VIEW_NAME, timezoneNameArray[idx], -1);
-		idx++;
+        if (timezoneAreaArray[idx])
+        {
+    	    gtk_list_store_append(timezoneAreaModel, &timezoneAreaIter);
+	   	    gtk_list_store_set(timezoneAreaModel, &timezoneAreaIter,
+                   			COL_VIEW_ID, idx,
+                        	COL_VIEW_NAME, timezoneNameArray[idx], -1);
+        }    
+        idx++;
 	}
 	label = gtk_label_new(_("Area"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -576,14 +594,15 @@ Time_Build_Objects(gboolean nonroot)
 	self.timezoneArea	= gtk_combo_box_new_with_model(GTK_TREE_MODEL(timezoneAreaModel));
 	timezoneAreaRend	= gtk_cell_renderer_text_new();
 	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(self.timezoneArea), 
-				timezoneAreaRend, 
-				TRUE);
+				timezoneAreaRend, TRUE);
 	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(self.timezoneArea), 
-				timezoneAreaRend,
-               			"text", COL_VIEW_NAME,
-               			NULL);
-	gtk_combo_box_set_active(GTK_COMBO_BOX (self.timezoneArea), selTimezoneAreaIndx);
-	gtk_tooltips_set_tip(tooltips, self.timezoneArea, _("Select your current timezone area here. The setting applies after the next login."), NULL);
+				timezoneAreaRend,"text", COL_VIEW_NAME,	NULL);
+    
+	gtk_combo_box_set_active(GTK_COMBO_BOX (self.timezoneArea), 
+                             area_index_to_selector_index (selTimezoneAreaIndx));
+	gtk_tooltips_set_tip(tooltips, self.timezoneArea, 
+                         _("Select your current timezone area here. "\
+                           "The setting applies after the next login."), NULL);
 	gtk_table_attach (GTK_TABLE (table), self.timezoneArea, 
 	                  1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(self.timezoneArea)), "changed",
@@ -593,20 +612,20 @@ Time_Build_Objects(gboolean nonroot)
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label,	0, 1, 2, 3,	GTK_FILL | GTK_EXPAND, 
 	                  0, 0, 0);
-	// timezone combobox
+	/* timezone combobox */
 	timezoneModel	= gtk_list_store_new(N_VIEW_COLUMNS,
 	                                     G_TYPE_INT, G_TYPE_STRING);
 	idx			= 0;
 	timezoneAreasPointer	= timezoneAreaArray[selTimezoneAreaIndx];
-	while(*timezoneAreasPointer != NULL) 
-	{
-		gtk_list_store_append(timezoneModel, &timezoneIter);
-		gtk_list_store_set(timezoneModel, &timezoneIter,
-					COL_VIEW_ID, idx,
-					COL_VIEW_NAME, *timezoneAreasPointer, -1);
-		timezoneAreasPointer++;
-		idx++;
-	}
+    if (timezoneAreasPointer)
+        while(timezoneAreasPointer[idx] != NULL) 
+    	{
+    		gtk_list_store_append(timezoneModel, &timezoneIter);
+    		gtk_list_store_set(timezoneModel, &timezoneIter,
+    					COL_VIEW_ID, idx,
+    					COL_VIEW_NAME, timezoneAreasPointer[idx], -1);
+    		idx++;
+    	}
 	self.timezone	= gtk_combo_box_new_with_model(GTK_TREE_MODEL(timezoneModel));
 	timezoneRend	= gtk_cell_renderer_text_new();
 	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(self.timezone),
@@ -614,11 +633,14 @@ Time_Build_Objects(gboolean nonroot)
 	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(self.timezone),
 	                               timezoneRend, "text", COL_VIEW_NAME, NULL);	
 	gtk_tooltips_set_tip (tooltips, self.timezone, 
-	                      _("Select your current timezone here. The setting applies after the next login."), NULL);
+	                      _("Select your current timezone here. "\
+                            "The setting applies after the next login."), NULL);
 	gtk_table_attach (GTK_TABLE (table), self.timezone,
 	                  1, 2, 2, 3, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+    
 	gtk_combo_box_set_active(GTK_COMBO_BOX (self.timezone), selTimezoneIndx);
-	
+
+ 
 	g_signal_connect_after (G_OBJECT (notebook), "switch-page",
 				G_CALLBACK (do_tabchange), NULL);
 	gtk_widget_show_all(notebook);
